unit Profiles;

interface

uses
  Windows;

const
  ROOTKEY              = 'Software\FireBird\TMSRemote';
  INIFILE              = 'TMSRemoteProfiles.ini';

type
  tConnectionProfile = record
    ProfileName        : String;

    TFIRInUse          : Boolean;
    TFIRIP             : String;
    TFIRPort           : word;
    TFIRTimeout        : dword;
    TFIRRetries        : dword;
    TFIRRemoteMode     : byte;
    TFIRPowerOnMode    : byte;
    TFIRPingIP         : String;
    TFIRPingMAC        : String;

    ToppyIP            : String;
    ToppyPort          : word;
    ToppyVFDPort       : word;
    BootTimeout        : dword;
    BootRetires        : dword;
    PwdHash            : dword;

    TMSRemoteTimeout   : dword;
  end;

var
  ConnectionProfile       : array[0..99] of tConnectionProfile;
  NrConnectionProfiles    : dword;
  CurrentConnectionProfile: dword;

function  ReadConnectionProfile(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;
function  ReadConnectionProfileREG(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;
function  ReadConnectionProfileINI(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;
procedure WriteConnectionProfile(ProfileIndex: byte; var Profile: tConnectionProfile);
procedure WriteConnectionProfileREG(ProfileIndex: byte; var Profile: tConnectionProfile);
procedure WriteConnectionProfileINI(ProfileIndex: byte; var Profile: tConnectionProfile);
procedure DeleteConnectionProfile(ProfileIndex: byte);
procedure DeleteConnectionProfileREG(ProfileIndex: byte);
procedure DeleteConnectionProfileINI(ProfileIndex: byte);
procedure DeleteConnectionProfileINIFile();
function  DoesINIProfileExist(): Boolean;
function  CurrentRootPath(): String;
function  GetININame(): String;

implementation

uses
  Registry, INIFiles, SysUtils;

function GetININame(): String;
begin
  Result := CurrentRootPath + INIFILE;
end;

function DoesINIProfileExist(): Boolean;
begin
  Result := FileExists(GetININame());
end;

function ReadConnectionProfile(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;
begin
  if DoesINIProfileExist() then
    Result := ReadConnectionProfileINI(ProfileIndex, Profile)
  else
    Result := ReadConnectionProfileREG(ProfileIndex, Profile)
end;

function ReadConnectionProfileREG(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;

var
  ProfileNameRoot      : String;
  Reg                  : tRegistry;

begin
  if DoesINIProfileExist() then
  begin
    Result := ReadConnectionProfileINI(ProfileIndex, Profile);
    exit;
  end;

  Result := false;

  Reg := tRegistry.Create();
  Reg.RootKey := HKEY_CURRENT_USER;
  Reg.OpenKey(ROOTKEY, true);

  ProfileNameRoot := format('ConnectionProfile_%2.2d_', [ProfileIndex + 1]);
  if Reg.ValueExists(ProfileNameRoot + 'Name') then
  begin
    Result := true;
    Profile.ProfileName := Reg.ReadString(ProfileNameRoot + 'Name');

    Profile.ToppyIP     := Reg.ReadString(ProfileNameRoot + 'ToppyIP');
    Profile.ToppyPort   := Reg.ReadInteger(ProfileNameRoot + 'ToppyPort');

    if Reg.ValueExists(ProfileNameRoot + 'ToppyVFDPort') then
      Profile.ToppyVFDPort := Reg.ReadInteger(ProfileNameRoot + 'ToppyVFDPort')
    else
      Profile.ToppyVFDPort := 1415;

    Profile.TFIRInUse   := Reg.ReadBool(ProfileNameRoot + 'TFIRInUse');
    Profile.TFIRIP      := Reg.ReadString(ProfileNameRoot + 'TFIRIP');
    Profile.TFIRPort    := Reg.ReadInteger(ProfileNameRoot + 'TFIRPort');
    Profile.TFIRTimeout := Reg.ReadInteger(ProfileNameRoot + 'TFIRTimeout');
    Profile.TFIRRetries := Reg.ReadInteger(ProfileNameRoot + 'TFIRRetries');
    Profile.TFIRRemoteMode := Reg.ReadInteger(ProfileNameRoot + 'TFIRRemoteMode');

    if Reg.ValueExists(ProfileNameRoot + 'TFIRPowerOnMode') then
      Profile.TFIRPowerOnMode := Reg.ReadInteger(ProfileNameRoot + 'TFIRPowerOnMode')
    else
      Profile.TFIRPowerOnMode := 0;

    if Reg.ValueExists(ProfileNameRoot + 'TFIRPingIP') then
      Profile.TFIRPingIP := Reg.ReadString(ProfileNameRoot + 'TFIRPingIP')
    else
      Profile.TFIRPingIP := Profile.ToppyIP;

    if Reg.ValueExists(ProfileNameRoot + 'TFIRPingIP') then
      Profile.TFIRPingMAC := Reg.ReadString(ProfileNameRoot + 'TFIRPingMAC')
    else if Reg.ValueExists(ProfileNameRoot + 'UseToppyMAC') then
      Profile.TFIRPingMAC := Reg.ReadString(ProfileNameRoot + 'ToppyMAC')
    else
      Profile.TFIRPingMAC := '00:00:00:00:00:00';

    Profile.BootTimeout := Reg.ReadInteger(ProfileNameRoot + 'BootTimeout');
    Profile.BootRetires := Reg.ReadInteger(ProfileNameRoot + 'BootRetires');

    Profile.TMSRemoteTimeout := Reg.ReadInteger(ProfileNameRoot + 'TMSRemoteTimeout');

    if Reg.ValueExists(ProfileNameRoot + 'PwdHash') then
      Profile.PwdHash := Reg.ReadInteger(ProfileNameRoot + 'PwdHash');
  end;

  Reg.CloseKey();
  Reg.Free();
end;

function ReadConnectionProfileINI(ProfileIndex: byte; var Profile: tConnectionProfile): boolean;

var
  ProfileNameRoot      : String;
  INI                  : tINIFile;

begin
  Result := false;

  INI := tINIFile.Create(GetININame());
  ProfileNameRoot := format('ConnectionProfile_%2.2d', [ProfileIndex + 1]);
  if INI.SectionExists(ProfileNameRoot) then
  begin
    Result := true;
    Profile.ProfileName := INI.ReadString(ProfileNameRoot, 'Name', '');

    Profile.ToppyIP     := INI.ReadString(ProfileNameRoot, 'ToppyIP', '127.0.0.1');
    Profile.ToppyPort   := INI.ReadInteger(ProfileNameRoot, 'ToppyPort', 1413);
    Profile.ToppyVFDPort := INI.ReadInteger(ProfileNameRoot, 'ToppyVFDPort', 1415);
    Profile.TFIRInUse   := INI.ReadBool(ProfileNameRoot, 'TFIRInUse', false);
    Profile.TFIRIP      := INI.ReadString(ProfileNameRoot, 'TFIRIP', '127.0.0.1');
    Profile.TFIRPort    := INI.ReadInteger(ProfileNameRoot, 'TFIRPort', 1414);
    Profile.TFIRTimeout := INI.ReadInteger(ProfileNameRoot, 'TFIRTimeout', 1);
    Profile.TFIRRetries := INI.ReadInteger(ProfileNameRoot, 'TFIRRetries', 3);
    Profile.TFIRRemoteMode := INI.ReadInteger(ProfileNameRoot, 'TFIRRemoteMode', 5);
    Profile.TFIRPowerOnMode := INI.ReadInteger(ProfileNameRoot, 'TFIRPowerOnMode', 0);

    if INI.ValueExists(ProfileNameRoot, 'TFIRPingIP') then
      Profile.TFIRPingIP := INI.ReadString(ProfileNameRoot, 'TFIRPingIP', '')
    else
      Profile.TFIRPingIP := Profile.ToppyIP;

    if INI.ValueExists(ProfileNameRoot, 'TFIRPingIP') then
      Profile.TFIRPingMAC := INI.ReadString(ProfileNameRoot, 'TFIRPingMAC', '')
    else if INI.ValueExists(ProfileNameRoot, 'UseToppyMAC') then
      Profile.TFIRPingMAC := INI.ReadString(ProfileNameRoot, 'ToppyMAC', '00:00:00:00:00:00');

    Profile.BootTimeout := INI.ReadInteger(ProfileNameRoot, 'BootTimeout', 25);
    Profile.BootRetires := INI.ReadInteger(ProfileNameRoot, 'BootRetires', 3);
    Profile.TMSRemoteTimeout := INI.ReadInteger(ProfileNameRoot, 'TMSRemoteTimeout', 120);
    Profile.PwdHash := INI.ReadInteger(ProfileNameRoot, 'PwdHash', 0);
  end;

  INI.Free();
end;

procedure WriteConnectionProfile(ProfileIndex: byte; var Profile: tConnectionProfile);
begin
  if DoesINIProfileExist() then
    WriteConnectionProfileINI(ProfileIndex, Profile)
  else
    WriteConnectionProfileREG(ProfileIndex, Profile);
end;

procedure WriteConnectionProfileREG(ProfileIndex: byte; var Profile: tConnectionProfile);

var
  ProfileNameRoot      : String;
  Reg                  : tRegistry;

begin

  Reg := tRegistry.Create();
  Reg.RootKey := HKEY_CURRENT_USER;
  Reg.OpenKey(ROOTKEY, true);

  ProfileNameRoot := format('ConnectionProfile_%2.2d_', [ProfileIndex + 1]);

  Reg.WriteString(ProfileNameRoot + 'Name', Profile.ProfileName);

  Reg.WriteString(ProfileNameRoot + 'ToppyIP', Profile.ToppyIP);
  Reg.WriteInteger(ProfileNameRoot + 'ToppyPort', Profile.ToppyPort);
  Reg.WriteInteger(ProfileNameRoot + 'ToppyVFDPort', Profile.ToppyVFDPort);

  Reg.WriteBool(ProfileNameRoot + 'TFIRInUse', Profile.TFIRInUse);
  Reg.WriteString(ProfileNameRoot + 'TFIRIP', Profile.TFIRIP);
  Reg.WriteInteger(ProfileNameRoot + 'TFIRPort', Profile.TFIRPort);
  Reg.WriteInteger(ProfileNameRoot + 'TFIRTimeout', Profile.TFIRTimeout);
  Reg.WriteInteger(ProfileNameRoot + 'TFIRRetries', Profile.TFIRRetries);
  Reg.WriteInteger(ProfileNameRoot + 'TFIRRemoteMode', Profile.TFIRRemoteMode);
  Reg.WriteInteger(ProfileNameRoot + 'TFIRPowerOnMode', Profile.TFIRPowerOnMode);
  Reg.WriteString(ProfileNameRoot + 'TFIRPingIP', Profile.TFIRPingIP);
  Reg.WriteString(ProfileNameRoot + 'TFIRPingMAC', Profile.TFIRPingMAC);

  Reg.WriteInteger(ProfileNameRoot + 'BootTimeout', Profile.BootTimeout);
  Reg.WriteInteger(ProfileNameRoot + 'BootRetires', Profile.BootRetires);
  Reg.WriteInteger(ProfileNameRoot + 'TMSRemoteTimeout', Profile.TMSRemoteTimeout);
  Reg.WriteInteger(ProfileNameRoot + 'PwdHash', Profile.PwdHash);

  Reg.CloseKey();
  Reg.Free();
end;

procedure WriteConnectionProfileINI(ProfileIndex: byte; var Profile: tConnectionProfile);

var
  ProfileNameRoot      : String;
  INI                  : tINIFile;

begin
  if not(DoesINIProfileExist()) or ((GetFileAttributes(pChar(GetININame())) and FILE_ATTRIBUTE_READONLY) = 0) then
  begin
    INI := tINIFile.Create(GetININame());

    ProfileNameRoot := format('ConnectionProfile_%2.2d', [ProfileIndex + 1]);

    INI.WriteString(ProfileNameRoot, 'Name', Profile.ProfileName);

    INI.WriteString(ProfileNameRoot, 'ToppyIP', Profile.ToppyIP);
    INI.WriteInteger(ProfileNameRoot, 'ToppyPort', Profile.ToppyPort);
    INI.WriteInteger(ProfileNameRoot, 'ToppyVFDPort', Profile.ToppyVFDPort);

    INI.WriteBool(ProfileNameRoot, 'TFIRInUse', Profile.TFIRInUse);
    INI.WriteString(ProfileNameRoot, 'TFIRIP', Profile.TFIRIP);
    INI.WriteInteger(ProfileNameRoot, 'TFIRPort', Profile.TFIRPort);
    INI.WriteInteger(ProfileNameRoot, 'TFIRTimeout', Profile.TFIRTimeout);
    INI.WriteInteger(ProfileNameRoot, 'TFIRRetries', Profile.TFIRRetries);
    INI.WriteInteger(ProfileNameRoot, 'TFIRRemoteMode', Profile.TFIRRemoteMode);
    INI.WriteInteger(ProfileNameRoot, 'TFIRPowerOnMode', Profile.TFIRPowerOnMode);
    INI.WriteString(ProfileNameRoot, 'TFIRPingIP', Profile.TFIRPingIP);
    INI.WriteString(ProfileNameRoot, 'TFIRPingMAC', Profile.TFIRPingMAC);

    INI.WriteInteger(ProfileNameRoot, 'BootTimeout', Profile.BootTimeout);
    INI.WriteInteger(ProfileNameRoot, 'BootRetires', Profile.BootRetires);

    INI.WriteInteger(ProfileNameRoot, 'TMSRemoteTimeout', Profile.TMSRemoteTimeout);

    INI.WriteInteger(ProfileNameRoot, 'PwdHash', Profile.PwdHash);

    INI.Free();
  end;
end;

procedure DeleteConnectionProfile(ProfileIndex: byte);
begin
  if DoesINIProfileExist() then
    DeleteConnectionProfileINI(ProfileIndex)
  else
    DeleteConnectionProfileREG(ProfileIndex);
end;

procedure DeleteConnectionProfileREG(ProfileIndex: byte);

var
  ProfileNameRoot      : String;
  Reg                  : tRegistry;

begin
  Reg := tRegistry.Create();
  Reg.RootKey := HKEY_CURRENT_USER;
  Reg.OpenKey(ROOTKEY, true);

  ProfileNameRoot := format('ConnectionProfile_%2.2d_', [ProfileIndex + 1]);

  if Reg.ValueExists(ProfileNameRoot + 'Name') then
  begin
    Reg.DeleteValue(ProfileNameRoot + 'Name');

    Reg.DeleteValue(ProfileNameRoot + 'ToppyIP');
    Reg.DeleteValue(ProfileNameRoot + 'ToppyPort');
    Reg.DeleteValue(ProfileNameRoot + 'ToppyVFDPort');

    Reg.DeleteValue(ProfileNameRoot + 'TFIRInUse');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRIP');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRPort');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRTimeout');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRRetries');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRRemoteMode');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRPowerOnMode');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRPingIP');
    Reg.DeleteValue(ProfileNameRoot + 'TFIRPingMAC');

    Reg.DeleteValue(ProfileNameRoot + 'BootTimeout');
    Reg.DeleteValue(ProfileNameRoot + 'BootRetires');

    Reg.DeleteValue(ProfileNameRoot + 'TMSRemoteTimeout');

    Reg.DeleteValue(ProfileNameRoot + 'PwdHash');

    Reg.DeleteValue(ProfileNameRoot + 'UseToppyMAC');
    Reg.DeleteValue(ProfileNameRoot + 'ToppyMAC');
  end;

  Reg.CloseKey();
  Reg.Free();
end;

procedure DeleteConnectionProfileINI(ProfileIndex: byte);

var
  ProfileNameRoot      : String;
  INI                  : tINIFile;

begin
  INI := tINIFile.Create(GetININame());

  ProfileNameRoot := format('ConnectionProfile_%2.2d', [ProfileIndex + 1]);

  if INI.SectionExists(ProfileNameRoot) then
    INI.EraseSection(ProfileNameRoot);

  INI.Free();
end;

procedure DeleteConnectionProfileINIFile();
begin
  if FileExists(GetININame()) then DeleteFile(GetININame());
end;

function CurrentRootPath(): String;
begin
  Result := IncludeTrailingBackslash(ExtractFilePath(paramstr(0)));
end;

end.

