#include                <stdlib.h>
#include                <string.h>
#include                <sys/socket.h>
#include                <arpa/inet.h>
#include                <unistd.h>
#include                <netinet/in.h>
#include                <fcntl.h>
#include                <errno.h>
#include                "Network.h"
#include                "TMSRemote.h"
#include                "websocket.h"
#include                "libFireBird.h"

#define ROOTDIR         "/ProgramFiles/Settings/" PROGRAM_NAME

int                     Settings_MaxPending;
int                     Settings_BufferSize;
int                     Settings_Port;
bool                    Authenticated = FALSE;
dword                   Settings_Password;
dword                   HiddenOsd = 0;
dword                   AliveTimer = 0;
bool                    HighSpeedUpdate;
dword                   HighSpeedUpdateTimeout = 0;


bool                    ModeWebSocket = FALSE;  // false: standard mode;
                                                // is set to true, if a valid WebSocket connection request is received
                                                // and reset to false, if connection is closed
unsigned char           wsHeaderBuffer[10];     // buffer for WebSocket frame header (send)

extern void WriteLog(char *s);
void HideOsd(bool Hide);

char                    LogPuffer[256];
#define DEBUG(...)      {sprintf(LogPuffer, __VA_ARGS__); WriteLog(LogPuffer);}

//-------------------------------------------------------------------------------------------------------
// Settings
//-------------------------------------------------------------------------------------------------------

void CreateRootDir(void)
{
  //Check & Create Folders
  HDD_TAP_PushDir();
  HDD_ChangeDir("/ProgramFiles");
  if(!TAP_Hdd_Exist("Settings")) TAP_Hdd_Create("Settings", ATTR_FOLDER);
  HDD_ChangeDir("Settings");
  if(!TAP_Hdd_Exist(PROGRAM_NAME)) TAP_Hdd_Create(PROGRAM_NAME, ATTR_FOLDER);
  HDD_TAP_PopDir();
}

void LoadSettings()
{
  INILOCATION           ProfileLocation;
  char                  Password[50];

  CreateRootDir();
  TAP_Hdd_ChangeDir(ROOTDIR);
  ProfileLocation = INIOpenFile("TMSRemote_Network.ini", PROGRAM_NAME);
  if(ProfileLocation > INILOCATION_NewFile)
  {
    Settings_MaxPending = INIGetInt ("MaxPending", 1, 1, 1);
    Settings_BufferSize = INIGetInt ("BufferSize", 65535,  1, 65535);
    Settings_Port = INIGetInt ("Port", NETWORKPORT,  1, 65535);

    //Read the password and build a simple hash
    INIGetString("Password", Password, "", sizeof(Password));
    if(Password[0])
      Settings_Password = CRC32(0xffffffff, Password, strlen(Password));
    else
      Settings_Password = 0;

    INICloseFile();
  }
  else  //load defaults
  {
    DEBUG("RMT: %s not found, using default settings", "TMSRemote_Network.ini");
    Settings_MaxPending    = 1;
    Settings_BufferSize    = 65535;
    Settings_Port          = NETWORKPORT;
    Settings_Password      = 0;
  }
  DEBUG("RMT: Listening on TCP port %d", Settings_Port);
}

//-------------------------------------------------------------------------------------------------------
// Socket functions
//-------------------------------------------------------------------------------------------------------

static int                    ServerSock = 0;
static int                    ClientSock = 0;
static struct sockaddr_in     echoclient;

void SetupSocketServer()
{
  struct sockaddr_in    echoserver;
  int                   flags;

  //Create the TCP socket
  if ((ServerSock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
  {
    DEBUG("RMT: Failed to create socket");
    return;
  }
  //DEBUG("RMT: Server socket ID = 0x%8.8x", ServerSock);

  //set non blocking mode
  flags = fcntl(ServerSock, F_GETFL, 0);
  fcntl(ServerSock, F_SETFL, flags | O_NONBLOCK);

  //Construct the server sockaddr_in structure
  memset(&echoserver, 0, sizeof(echoserver));       // Clear struct
  echoserver.sin_family = AF_INET;                  // Internet/IP
  echoserver.sin_addr.s_addr = htonl(INADDR_ANY);   // Incoming addr
  echoserver.sin_port = htons(Settings_Port);       // server port

  //DEBUG("RMT: Binding server socket on port %d...", Settings_Port);
  // Bind the server socket
  if(bind(ServerSock, (struct sockaddr *) &echoserver, sizeof(echoserver)) < 0)
  {
    DEBUG("RMT: Failed to bind the server socket");
    return;
  }

  //DEBUG("RMT: Listening on server socket...");
  // Listen on the server socket
  if(listen(ServerSock, Settings_MaxPending) < 0)
  {
    DEBUG("RMT: Failed to listen on server socket");
    return;
  }
}

bool CloseSocket(void)
{
  if(ClientSock > 0)
  {
    DEBUG("RMT: Closing connection to client");
    close(ClientSock);
    Authenticated = FALSE;
    ClientSock = 0;
    AliveTimer = 0;
    HideOsd(FALSE);
    ModeWebSocket = FALSE;

    return TRUE;
  }

  return FALSE;
}

int CheckSocket(byte *Buffer)
{
  int                   received = -1;
  unsigned int          clientlen = sizeof(echoclient);

  if(!ServerSock) return 0;

  // Wait for client connection
  if(ClientSock <= 0)
  {
    if ((ClientSock = accept(ServerSock, (struct sockaddr *) &echoclient, &clientlen)) < 0)
    {
      if(errno == EAGAIN) return 0;   //nothing waiting

      DEBUG("RMT: Failed to accept client connection, error = %d", ClientSock);
      return 0;
    }
    DEBUG("RMT: Client connected: ID = 0x%8.8x, %s", ClientSock, inet_ntoa(echoclient.sin_addr));
    AliveTimer = TAP_GetTick() + ALIVETIMEOUT;
    Authenticated = FALSE;
    ModeWebSocket = FALSE;
  }

  // Receive message
  if ((ClientSock > 0) && ((received = recv(ClientSock, Buffer, Settings_BufferSize, MSG_DONTWAIT)) < 0))
  {
    if(errno == EAGAIN) return 0;   //nothing waiting

    DEBUG("RMT: Failed to receive bytes from client, error = 0x%8.8x", errno);
    return 0;
  }

  if(received == 0) CloseSocket();

  // if in WebSocket mode: decode WebSocket frame to binary data
  if(ModeWebSocket) received = ws_decode_frame(Buffer, Buffer);

  //if(received) DumpMemory(Buffer, received, 16);

  return received;
}

bool SendSocket(byte *Buffer, int BufferLen)
{
  if(ServerSock && (ClientSock > 0))
  {
    // if in WebSocket mode: create and send WebSocket frame header first
    if(ModeWebSocket)
    {
      int header_len = ws_create_frame_header(BufferLen, wsHeaderBuffer, wsft_binary, ws_unmasked);

      if(header_len <= (int) sizeof(wsHeaderBuffer))
      {
        if(send(ClientSock, wsHeaderBuffer, header_len, 0) != header_len)
        {
          DEBUG("RMT: Failed to send websocket frame header to client");
          return FALSE;
        }
      }
      else
      {
        DEBUG("RMT: SendSocket: buffer size to large for frame buffer");
        return FALSE;
      }
    }

    if(send(ClientSock, Buffer, BufferLen, 0) != BufferLen)
    {
      DEBUG("RMT: Failed to send bytes to client");
      return FALSE;
    }
    return TRUE;
  }

  return FALSE;
}

void ShutdownSocketServer()
{
  if(ServerSock)
  {
    DEBUG("RMT: Closing server socket");
    close(ServerSock);
    ServerSock = 0;
  }
}

bool ActiveConnection(void)
{
  return ((ServerSock > 0) && (ClientSock > 0));
}

//-------------------------------------------------------------------------------------------------------
// State machine
//-------------------------------------------------------------------------------------------------------
static byte            *RcvBuffer = NULL;
int                     RcvBufferPtIn = 0;
int                     RcvBufferPtOut = 0;
static byte           *_keyboardStart = NULL;  //The content behind this pointer is TRUE if the Topfield Keyboard is shown
static byte             LastKeyboardStart = 0;

bool Network_IsAnyConnection(void)
{
  return (Authenticated && (ClientSock > 0));
}

void Network_Close(void)
{
  DEBUG("RMT: Shutdown event");
  CloseSocket();
  ShutdownSocketServer();
  if(RcvBuffer)
  {
    TAP_MemFree(RcvBuffer);
    RcvBuffer = NULL;
  }
}

void HideOsd(bool Hide)
{
  if(Hide)
  {
    HiddenOsd = 1;
    TAP_Osd_SetTransparency(BASE_PLANE, 0);
  }
  else
  {
    TAP_Osd_SetTransparency(BASE_PLANE, 255 - TAP_GetSystemVar(SYSVAR_OsdAlpha));
    HiddenOsd = 0;
  }
}

bool Network_IsOsdHidden(void)
{
  return (HiddenOsd != 0);
}

void SendMessage(dword MessageID)
{
  tMacroBlock           Data;

  Data.BlockIndex = 2000;     //Special macroblock ID
  Data.RLELength  =    1;
  Data.RLEData[0] = MessageID;
  SendSocket((byte*)&Data, 12);
}

dword Network_EventHandler(word event, dword param1, dword param2)
{
  static bool           doNotReenter = FALSE;
  static bool           Init = TRUE;
  static bool           Exit = FALSE;

  UNUSED(param2);

  //Don't do anything if the network server has been closed
  if(Exit) return param1;

  if(doNotReenter) return param1;

  doNotReenter = TRUE;

  //First initialization
  if(Init)
  {
    RcvBuffer = TAP_MemAlloc(RCVBUFFERSIZE);
    RcvBufferPtIn = 0;
    RcvBufferPtOut = 0;
    HighSpeedUpdate = FALSE;
    HighSpeedUpdateTimeout = 0;

    _keyboardStart = (byte*)TryResolve("_keyboardStart");
    if(!_keyboardStart)
    {
      WriteLog("Failed to locate _keyboardStart. DirectMode for Topfield keyboard is not available.");
    }

    LoadSettings();
    SetupSocketServer();
    Init = FALSE;
  }

  //An event controlled shutdown
  if(event == EVT_STOP && param1 < 2)
  {
    Exit = TRUE;
    Network_SendDirectOff();
    HideOsd(FALSE);
    Network_Close();
    doNotReenter = FALSE;
    return param1;
  }

  //Manage the network interface
  if(event == EVT_IDLE)
  {
    if(_keyboardStart && (*_keyboardStart != LastKeyboardStart))
    {
      LastKeyboardStart = *_keyboardStart;
      if(LastKeyboardStart)
        Network_SendDirectOn();
      else
        Network_SendDirectOff();
    }

    if(TAP_GetTick() > HighSpeedUpdateTimeout) HighSpeedUpdate = FALSE;

    RcvBufferPtIn += CheckSocket(&RcvBuffer[RcvBufferPtIn]);

    while (RcvBufferPtIn > RcvBufferPtOut)
    {
      switch(RcvBuffer[RcvBufferPtOut])
      {
        case Cmd_Alive:
        {
          AliveTimer = TAP_GetTick() + ALIVETIMEOUT;

          RcvBufferPtOut += sizeof(tCmd_Alive);
          break;
        }

        case Cmd_Authenticate:
        {
          tCmd_Authenticate *Data = (tCmd_Authenticate*)&RcvBuffer[RcvBufferPtOut];

          if(Data->CRC32 == Settings_Password)
          {
            Authenticated = TRUE;
            DEBUG("RMT: Authentication successful");
            HighSpeedUpdate = TRUE;
            HighSpeedUpdateTimeout = TAP_GetTick() + HIGHSPEEDUPDATETIMEOUT;
          }
          else
          {
            DEBUG("RMT: Authentication failed");
            SendMessage(MID_AuthFailed);
            CloseSocket();
          }
          RcvBufferPtOut += sizeof(tCmd_Authenticate);

          break;
        }

        case Cmd_RefreshAll:
        {
          if(!Authenticated)
          {
            DEBUG("RMT: Unauthorized access");
            CloseSocket();
          }

          RcvBufferPtOut += sizeof(tCmd_RefreshAll);
          break;
        }

        case Cmd_KeyEvent:
        {
          tCmd_KeyEvent *Data = (tCmd_KeyEvent*)&RcvBuffer[RcvBufferPtOut];

          if(Authenticated)
          {
            HighSpeedUpdate = TRUE;
            HighSpeedUpdateTimeout = TAP_GetTick() + HIGHSPEEDUPDATETIMEOUT;

            HideOsd(HiddenOsd);
            if((Data->Key == 0x1000a) || (Data->Key == 0x1001000a))
            {
              Shutdown(TaskPower);
            }
            else
              TAP_GenerateEvent(EVT_KEY, Data->Key, 0);
          }
          else
          {
            DEBUG("RMT: Unauthorized access");
            CloseSocket();
          }

          RcvBufferPtOut += sizeof(tCmd_KeyEvent);
          break;
        }

        case Cmd_ASCIIEvent:
        {
          tCmd_ASCIIEvent *Data = (tCmd_ASCIIEvent*)&RcvBuffer[RcvBufferPtOut];

          if(Authenticated)
          {
            HighSpeedUpdate = TRUE;
            HighSpeedUpdateTimeout = TAP_GetTick() + HIGHSPEEDUPDATETIMEOUT;

            HideOsd(HiddenOsd);

            if(LastKeyboardStart)
            {
              Data->ASCIICode += 0x20000;
              switch(Data->ASCIICode)
              {
                case 0x2007f: Data->ASCIICode = 0x2f805; break;     //Del
                case 0x20124: Data->ASCIICode = 0x2f806; break;     //Pos1
                case 0x20123: Data->ASCIICode = 0x2f807; break;     //End
                case 0x20121: Data->ASCIICode = 0x2f800; break;     //PgUp
                case 0x20122: Data->ASCIICode = 0x2f801; break;     //PgDown

                case 0x20127: Data->ASCIICode = RKEY_Next; break;   //->
                case 0x20125: Data->ASCIICode = RKEY_Prev; break;   //<-
              }

              TAP_GenerateEvent(EVT_KEY, Data->ASCIICode, 0);
            }
            else
            {
              HDD_TAP_SendEvent(0, FALSE, EVT_TMSREMOTEASCII, Data->ASCIICode, 0);
            }
          }
          else
          {
            DEBUG("RMT: Unauthorized access");
            CloseSocket();
          }

          RcvBufferPtOut += sizeof(tCmd_KeyEvent);
          break;
        }

        case Cmd_HideOsd:
        {
          tCmd_HideOsd *Data = (tCmd_HideOsd*)&RcvBuffer[RcvBufferPtOut];

          HideOsd(Data->HideOsd);

          RcvBufferPtOut += sizeof(tCmd_HideOsd);

          break;
        }

        case Cmd_Reboot:
        {
          if(!Authenticated)
          {
            DEBUG("RMT: Unauthorized access");
            CloseSocket();
          }
          else
          {
            CloseSocket();
            TAP_Reboot();
          }

          RcvBufferPtOut += sizeof(tCmd_Reboot);
          break;
        }

        case Cmd_VideoFrame:
        {
          VideoCapture();

          RcvBufferPtOut += sizeof(tCmd_VideoFrame);
          break;
        }

        // WebSocket close by client handshake; see websocket.c: (opcode == wsft_close)
        case Cmd_WsClose:
        {
          if(!Authenticated)
          {
            DEBUG("RMT: Unauthorized access");
          }

          CloseSocket();
          RcvBufferPtOut += sizeof(tCmd_WsClose);

          break;
        }

        // first byte is char 'G': this may be a WebSocket handshake: "GET /tmsremote HTTP/1.1"
        case Cmd_WsConnect:
        {
          tRequestHeader header;

          int size = ws_parse_header((char*)&RcvBuffer[RcvBufferPtOut] , RcvBufferPtIn - RcvBufferPtOut, &header);

          if(size == 0)
          {
            // error in WebSocket header: deny connection
            RcvBufferPtOut = RcvBufferPtIn;
            CloseSocket();
          }
          else
          {
            // send websocket handshake response
            SendSocket((byte*)header.response, strlen(header.response));
            RcvBufferPtOut += size;
            ModeWebSocket = TRUE;
          }
          break;
        }

        default:
          RcvBufferPtOut++;
          break;
      }
    }
    RcvBufferPtIn = 0;
    RcvBufferPtOut = 0;

    if(AliveTimer && (TAP_GetTick() > AliveTimer))
    {
      DEBUG("RMT: ALIVE timeout reached");
      CloseSocket();
    }
  }

  doNotReenter = FALSE;
  return param1;
}

void Network_SendDirectOn(void)
{
  SendMessage(MID_DirectOn);
}

void Network_SendDirectOff(void)
{
  SendMessage(MID_DirectOff);
}
